@@ -35,6 +35,11 @@ sub deleteNotUsedTokens {
   my %usedSymbols;
   @usedSymbols{@usedSymbols} = ();
 
+  for (@{$self->{GRAMMAR}{DUMMY}}) {
+    delete $usedSymbols{$_};
+    delete $termDef->{$_};
+  }
+
   for my $token (keys %$term) {
     delete $term->{$token} unless exists $usedSymbols{$token};
   }
@@ -105,9 +110,16 @@ sub makeLexer {
   for my $t (@termdef) {
     if ($termdef{$t}[2] eq 'REGEXP') {
       my $reg = $termdef{$t}[0];
-      $reg =~ s{^/}{/\\G};
+      $reg =~ s{^/}{/\\G}; # add \G at the begining of the regexp
       $DEFINEDTOKENS .= << "EORT";
       ${reg}gc and return ('$t', \$1);
+EORT
+    }
+    elsif ($termdef{$t}[2] eq 'CONTEXTUAL_REGEXP') {
+      my $reg = $termdef{$t}[0];
+      $reg =~ s{^/}{/\\G}; # add \G at the begining of the regexp
+      $DEFINEDTOKENS .= << "EORT";
+      \$self->expects('$t') and ${reg}gc and return ('$t', \$1);
 EORT
     }
     elsif ($termdef{$t}[2] eq 'LITERAL') { # %token without regexp or code definition
@@ -120,7 +132,7 @@ EORT
 EORT
     }
     elsif ($termdef{$t}[2] eq 'CODE') { # token definition is code
-      $DEFINEDTOKENS .= $termdef{$t}[0];
+      $DEFINEDTOKENS .= "$termdef{$t}[0]\n";
     }
   }
 
